namespace LibForTests
{
    public class NumpyNet
    {
        public static sbyte[] OneDimensionInt8Array(sbyte[] array) => array;
        public static short[] OneDimensionInt16Array(short[] array) => array;
        public static int[] OneDimensionInt32Array(int[] array) => array;
        public static long[] OneDimensionInt64Array(long[] array) => array;

        public static byte[] OneDimensionUInt8Array(byte[] array) => array;
        public static ushort[] OneDimensionUInt16Array(ushort[] array) => array;
        public static uint[] OneDimensionUInt32Array(uint[] array) => array;
        public static ulong[] OneDimensionUInt64Array(ulong[] array) => array;

        public static float[] OneDimensionFloat32Array(float[] array) => array;
        public static double[] OneDimensionFloat64Array(double[] array) => array;

        public static string[] OneDimensionStringArray(string[] array) => array;
    }
}
